"""Test Jupyter find/replace"""


INITIAL_CELLS = ["hello", "hellohello", "abc", "ello"]


def test_find_and_replace(notebook_frontend):
    """ test find and replace on all the cells """
    notebook_frontend.populate(INITIAL_CELLS)

    find_str = "ello"
    replace_str = "foo"

    notebook_frontend.wait_for_condition(lambda: notebook_frontend.get_cells_contents() == INITIAL_CELLS, timeout=120, period=5)

    # replace the strings
    notebook_frontend.find_and_replace(index=0, find_txt=find_str, replace_txt=replace_str)

    # check content of the cells
    notebook_frontend.wait_for_condition(
        lambda: notebook_frontend.get_cells_contents() == [
            s.replace(find_str, replace_str) for s in INITIAL_CELLS
        ]
    )



"""Test readonly notebook saved and renamed"""


from .utils import EDITOR_PAGE
from playwright.sync_api import sync_playwright, TimeoutError as PlaywrightTimeoutError


def save_as(nb):
    JS = '() => Jupyter.notebook.save_notebook_as()'
    return nb.evaluate(JS, page=EDITOR_PAGE)

def get_notebook_name(nb):
    JS = '() => Jupyter.notebook.notebook_name'
    return nb.evaluate(JS, page=EDITOR_PAGE)

def set_notebook_name(nb, name):
    JS = f'() => Jupyter.notebook.rename("{name}")'
    nb.evaluate(JS, page=EDITOR_PAGE)


def test_save_notebook_as(notebook_frontend):
    set_notebook_name(notebook_frontend, name="nb1.ipynb")

    notebook_frontend.locate('#notebook_name', page=EDITOR_PAGE)

    assert get_notebook_name(notebook_frontend) == "nb1.ipynb"

    # Wait for Save As modal, save
    save_as(notebook_frontend)
    notebook_frontend.wait_for_selector('.save-message', page=EDITOR_PAGE)

    # TODO: Add a function for locator assertions to FrontendElement
    locator_element = notebook_frontend.locate_and_focus('//input[@data-testid="save-as"]', page=EDITOR_PAGE)
    locator_element.wait_for('visible')

    notebook_frontend.insert_text('new_notebook.ipynb', page=EDITOR_PAGE)
    notebook_frontend.try_click_selector('//html//body//div[8]//div//div//div[3]//button[2]', page=EDITOR_PAGE)
    
    locator_element.expect_not_to_be_visible()

    assert get_notebook_name(notebook_frontend) == "new_notebook.ipynb"
    assert "new_notebook.ipynb" in notebook_frontend.get_page_url(page=EDITOR_PAGE)



"""Test clipboard functionality"""


from .utils import EDITOR_PAGE, validate_dualmode_state


INITIAL_CELLS = ['', 'print("a")', 'print("b")', 'print("c")']


def test_dualmode_clipboard(prefill_notebook):
    notebook_frontend = prefill_notebook(INITIAL_CELLS)
    _, a, b, c = INITIAL_CELLS
    for i in range(1, 4):
        notebook_frontend.execute_cell(i)

    # Copy/paste/cut
    num_cells = len(notebook_frontend.cells)
    assert notebook_frontend.get_cell_contents(1) == a  # Cell 1 is a

    notebook_frontend.focus_cell(1)
    notebook_frontend.press("x", EDITOR_PAGE)  # Cut
    validate_dualmode_state(notebook_frontend, 'command', 1)
    assert notebook_frontend.get_cell_contents(1) == b  # Cell 2 is now where cell 1 was
    assert len(notebook_frontend.cells) == num_cells - 1  # A cell was removed

    notebook_frontend.focus_cell(2)
    notebook_frontend.press("v", EDITOR_PAGE)  # Paste
    validate_dualmode_state(notebook_frontend, 'command', 3)
    assert notebook_frontend.get_cell_contents(3) == a  # Cell 3 has the cut contents
    assert len(notebook_frontend.cells) == num_cells  # A cell was added

    notebook_frontend.press("v", EDITOR_PAGE)  # Paste
    validate_dualmode_state(notebook_frontend, 'command', 4)
    assert notebook_frontend.get_cell_contents(4) == a  # Cell a has the cut contents
    assert len(notebook_frontend.cells) == num_cells + 1  # A cell was added

    notebook_frontend.focus_cell(1)
    notebook_frontend.press("c", EDITOR_PAGE)  # Copy
    validate_dualmode_state(notebook_frontend, 'command', 1)
    assert notebook_frontend.get_cell_contents(1) == b  # Cell 1 is b

    notebook_frontend.focus_cell(2)
    notebook_frontend.press("c", EDITOR_PAGE)  # Copy
    validate_dualmode_state(notebook_frontend, 'command', 2)
    assert notebook_frontend.get_cell_contents(2) == c  # Cell 2 is c

    notebook_frontend.focus_cell(4)
    notebook_frontend.press("v", EDITOR_PAGE)  # Paste
    validate_dualmode_state(notebook_frontend, 'command', 5)
    assert notebook_frontend.get_cell_contents(2) == c  # Cell 2 has the copied contents
    assert notebook_frontend.get_cell_contents(5) == c  # Cell 5 has the copied contents
    assert len(notebook_frontend.cells) == num_cells + 2  # A cell was added

    notebook_frontend.focus_cell(0)
    notebook_frontend.press('v', EDITOR_PAGE, ['Shift'])  # Paste
    validate_dualmode_state(notebook_frontend, 'command', 0)
    assert notebook_frontend.get_cell_contents(0) == c  # Cell 0 has the copied contents
    assert len(notebook_frontend.cells) == num_cells + 3  # A cell was added



"""Test the notification area and widgets"""


import pytest
from .utils import EDITOR_PAGE


def get_widget(notebook, name):
    return notebook.evaluate(
        f"() => {{ return IPython.notification_area.get_widget('{name}') !== undefined }}",
        page=EDITOR_PAGE
    )


def widget(notebook, name):
    return notebook.evaluate(
        f"() => {{ return IPython.notification_area.widget('{name}') !== undefined }}",
        page=EDITOR_PAGE
    )


def new_notification_widget(notebook, name):
    return notebook.evaluate(
        f"() => {{ return IPython.notification_area.new_notification_widget('{name}') !== undefined }}",
        page=EDITOR_PAGE
    )


def widget_has_class(notebook, name, class_name):
    return notebook.evaluate(
        f"""() => {{
        var w = IPython.notification_area.get_widget('{name}');
        return w.element.hasClass('{class_name}'); }}
        """,
        page=EDITOR_PAGE
    )


def widget_message(notebook, name):
    return notebook.evaluate(
        f"""() => {{
        var w = IPython.notification_area.get_widget('{name}');
        return w.get_message(); }}
        """,
        page=EDITOR_PAGE
    )


def test_notification(notebook_frontend):
    # check that existing widgets are there
    assert get_widget(notebook_frontend, "kernel") and widget(notebook_frontend, "kernel"),\
        "The kernel notification widget exists"
    assert get_widget(notebook_frontend, "notebook") and widget(notebook_frontend, "notebook"),\
        "The notebook notification widget exists"

    # try getting a non-existent widget
    with pytest.raises(Exception):
        get_widget(notebook_frontend, "foo")

    # try creating a non-existent widget
    assert widget(notebook_frontend, "bar"), "widget: new widget is created"

    # try creating a widget that already exists
    with pytest.raises(Exception):
        new_notification_widget(notebook_frontend, "kernel")

    # test creating 'info', 'warning' and 'danger' messages
    for level in ("info", "warning", "danger"):
        notebook_frontend.evaluate(
            f"""
            var tnw = IPython.notification_area.widget('test');
            tnw.{level}('test {level}');
            """,
            page=EDITOR_PAGE
        )
        notebook_frontend.wait_for_selector("#notification_test", page=EDITOR_PAGE)

        assert widget_has_class(notebook_frontend, "test", level), f"{level}: class is incorrect"
        assert widget_message(notebook_frontend, "test") == f"test {level}", f"{level}: message is incorrect"

    # test message timeout
    notebook_frontend.evaluate(
        """
        var tnw = IPython.notification_area.widget('test');
        tnw.set_message('test timeout', 1000);
        """,
        page=EDITOR_PAGE
    )
    notebook_frontend.wait_for_selector("#notification_test", page=EDITOR_PAGE)
    assert widget_message(notebook_frontend, "test") == "test timeout", "timeout: message is incorrect"

    notebook_frontend.wait_for_selector("#notification_test", EDITOR_PAGE, state='hidden')
    assert widget_message(notebook_frontend, "test") == "", "timeout: message was not cleared"

    # test click callback
    notebook_frontend.evaluate(
        """
        var tnw = IPython.notification_area.widget('test');
        tnw._clicked = false;
        tnw.set_message('test click', undefined, function () {
            tnw._clicked = true;
            return true;
        });
        """,
        page=EDITOR_PAGE
    )
    notebook_frontend.locate("#notification_test", page=EDITOR_PAGE)
    assert widget_message(notebook_frontend, "test") == "test click", "callback: message is correct"

    notebook_frontend.locate("#notification_test", page=EDITOR_PAGE).click()
    notebook_frontend.wait_for_condition(
        lambda: notebook_frontend.evaluate(
            '() => { return IPython.notification_area.widget("test")._clicked; }', page=EDITOR_PAGE
        )
    )

    notebook_frontend.wait_for_selector("#notification_test", EDITOR_PAGE, state='hidden')
    assert widget_message(notebook_frontend, "test") == "", "callback: message was not cleared"
